/*
 * Copyright 2021-2024 Hewlett Packard Enterprise Development LP
 * Other additional copyright holders may be indicated within.
 *
 * The entirety of this work is licensed under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "chpl/uast/Scan.h"

#include "chpl/uast/Builder.h"

namespace chpl {
namespace uast {


std::string Scan::dumpChildLabelInner(int i) const {
  if (i == opChildNum_) {
    return "op";
  } else if (i == iterandChildNum_) {
    return "iterand";
  }

  return Call::dumpChildLabelInner(i);
}

owned<Scan> Scan::build(Builder* builder, Location loc,
                        owned<AstNode> op,
                        owned<AstNode> iterand) {
  AstList children;
  children.push_back(std::move(op));
  children.push_back(std::move(iterand));

  Scan* ret = new Scan(std::move(children));
  builder->noteLocation(ret, loc);
  return toOwned(ret);
}


} // namespace uast
} // namespace chpl
