/*
 * Copyright 2020-2024 Hewlett Packard Enterprise Development LP
 * Copyright 2004-2019 Cray Inc.
 * Other additional copyright holders may be indicated within.
 *
 * The entirety of this work is licensed under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <map_field.h>

namespace chapel {

  MapFieldGenerator::MapFieldGenerator(const FieldDescriptor* descriptor)
      : FieldGeneratorBase(descriptor) {
  }

  MapFieldGenerator::~MapFieldGenerator() {
  }

  void MapFieldGenerator::GenerateMembers(Printer* printer) {
    const FieldDescriptor* key_descriptor =
        descriptor_->message_type()->FindFieldByName("key");
    const FieldDescriptor* value_descriptor =
        descriptor_->message_type()->FindFieldByName("value");
    variables_["key_type_name"] = type_name(key_descriptor);
    variables_["value_type_name"] = type_name(value_descriptor);
    printer->Print(variables_,
      "var $name$: map($key_type_name$, $value_type_name$);\n");
  }

}  // namespace chapel
